#include "mainwindow.h"
#include "ui_mainwindow.h"

double x1min,x1max,x2min,x2max;
double psi,Goo,r;
bool polarField, keepTackField, blackWhite ;
int q;


MainWindow::MainWindow(QWidget *parent) :  QMainWindow(parent),
      ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    x1min=-100;x1max=100;
    x2min=-100;x2max=100;
    R=new repere(this,ui->graphicsView,x1min,x1max,x2min,x2max);
    Init();
    Draw();
}


void MainWindow::Init()
{
    polarField=ui->polar_checkBox->isChecked();
    keepTackField=ui->keepTack_checkBox->isChecked();
    blackWhite=ui->blackWhite_checkBox->isChecked();
    psi=0.01*2*M_PI*ui->PsiScrollBar->value();
    Goo=0.01*M_PI*ui->Goo_ScrollBar->value();
    r=1.0*ui->r_ScrollBar->value();
    R->Clean();
    q=0;
}

void MainWindow::evalf(bool &pres, double &f1, double &f2, double x1, double x2)
{    double zeta=M_PI/3; // ou M_PI/6
     double theta=-(2/M_PI)*Goo*atan(x2/r);
     if ((q!=0)&(2*abs(x2)>r)) q=Sign(x2);  //q=0 : sans hysteresis
     //QPen pen1;  pen1.setWidth(1); pen1.setColor(Qt::black);
     //R->DrawArrow(x1,x2,8*cos(Gamma),8*sin(Gamma),0.1,pen1);
     //qDebug() << "Gamma=[ "<<Gamma.inf<<","<<Gamma.sup<<"].";
     bool b1=(cos(psi-theta)+cos(zeta)<0); //modeindirect
     bool b2=(abs(x2)-r<0);
     bool b3=(cos(psi)+cos(zeta)<0); //ligne face vent
     pres=false;
     if (b1&polarField)
     {  //if (blackWhite) {pen1.setColor(Qt::black);pen1.setWidth(2); }
        //else pen1.setColor(Qt::magenta);
         if (q==0) theta=M_PI+psi-zeta*Sign(x2); //+Error_cap;
         else theta=M_PI+psi-zeta*q;
         pres=true;
        //R->DrawArrow(x1,x2,8*f1,8*f2,0.1,pen1);
     }
     if ((!b1)&(b2&b3)&(keepTackField))
     {  //if (blackWhite) {pen1.setColor(Qt::black);pen1.setWidth(2); }
        //else pen1.setColor(Qt::red);
         if (q==0) theta=M_PI+psi-zeta*Sign(x2); //+Error_cap;
         else theta=M_PI+psi-zeta*q;
         pres=true;
        //R->DrawArrow(x1,x2,8*f1,8*f2,0.1,pen1);
     }
     f1=cos(theta); f2=sin(theta);
};



void MainWindow::Draw()
{
  QPen pen1;  pen1.setStyle(Qt::SolidLine); pen1.setWidth(1); pen1.setColor(Qt::black);
  for (double x1=x1min;x1<=x1max;x1=x1+10)
  for (double x2=x2min;x2<=x2max;x2=x2+10)
  if (abs(x2) > 0.1)
  {      double f1,f2;
         bool tack=false;
         evalf(tack,f1,f2,x1,x2);
         if (tack==true) pen1.setWidth(2);  else pen1.setWidth(1);
         R->DrawArrow(x1,x2,8*f1,8*f2,0.1,pen1);
  }
  pen1.setStyle(Qt::SolidLine); pen1.setWidth(6);
  if (blackWhite) pen1.setColor(Qt::darkGray); else  pen1.setColor(Qt::red);
  R->DrawArrow(50,60,40*cos(psi),40*sin(psi),0.1,pen1);
  pen1.setStyle(Qt::SolidLine); pen1.setWidth(3);
  if (blackWhite) pen1.setColor(Qt::gray); else pen1.setColor(Qt::blue);
  R->DrawLine(x1min,-r,x1max,-r,pen1);
  R->DrawLine(x1min,r,x1max, r,pen1);
  pen1.setColor(Qt::black);
  R->DrawArrow(x1min,0,x1max-x1min,0,0.01,pen1);
}

MainWindow::~MainWindow()
{
    delete ui;
}


void MainWindow::on_SaveButton_clicked()  {R->Save("field");}


void MainWindow::on_PsiScrollBar_valueChanged(int value)       {Init(); Draw();}
void MainWindow::on_Goo_ScrollBar_valueChanged(int value)      {Init(); Draw();}
void MainWindow::on_r_ScrollBar_valueChanged(int value)        {Init(); Draw();}
void MainWindow::on_polar_checkBox_stateChanged(int )          {Init(); Draw();}
void MainWindow::on_keepTack_checkBox_stateChanged(int )       {Init(); Draw();}
void MainWindow::on_blackWhite_checkBox_stateChanged(int )     {Init(); Draw();}

void MainWindow::on_allWindButton_clicked()
{
    Init();
    for (psi=0;psi<2*M_PI;psi=psi+0.2)
     Draw();
}

void MainWindow::on_simuButton_clicked()
{
    Init();
    Draw();
    q=1;
    bool pres;
    double f1,f2;
    double x1=x1min;
    double x2=x2min+(x2max-x2min)*rand()/RAND_MAX;
    double dt=1;
    QPen pen1;  pen1.setStyle(Qt::SolidLine); pen1.setWidth(2); pen1.setColor(Qt::magenta);

    for (double t=0;t<500;t=t+dt)
    {
        evalf(pres, f1, f2, x1, x2);
        if (pres) pen1.setColor(Qt::yellow); else pen1.setColor(Qt::green);
        double x1p=x1+dt*f1;
        double x2p=x2+dt*f2;
        R->DrawLine(x1,x2,x1p,x2p,pen1);
        x1=x1p;x2=x2p;

    }
}
